function zz = zlogd( sv, c, L, P, verbose )
% z = zlogd( sv, c, L, P, verbose )
% evaluates logarithmic derivative of zeta( sv( i, j ), c ) where c is parameter
% in map, sv is vector or matrix of complex powers in zeta sum, L is 
% number of levels and P number of basis functions. 
% Optionally returns the total CPU time required, if requested, in t.
% Example:
% z = zlogd( [ 0, 0.4, 0.8 ], -20, 4, 2 )
% produces 
% z = ( -4.36177246144466  11.19334327044841   1.15933325323951 )
% + 1.0e-15 * ( -0.21170594678626  -0.08502987568704  -0.01988155972457 ) * i

 % Set useful input values for unspecified arguments.

 % Parameter s in zeta function.
 if( nargin < 1 )
  sv = -1.0 + sqrt( -1.0 ) * 2.0
 end

 % Parameter c in z^2 + c map.
 if( nargin < 2 )
  c = -10;
 end

 % Number of levels to iterate around Julia set.
 if( nargin < 3 )
  L = 2;
 end

 % Number of intervals = number of blocks per dimension of transfer matrix.
 N = 2^L;
 
 % Number of powers per interval = block size of transfer matrix.
 if( nargin < 4 )
  P = 2;
 end
 
 % Output control: 0 is no output, more output for higher values.
 if( nargin < 5 )
  verbose = 0;
 end
 
 % Here we should build a quadrature formula that gives desired accuracy
 % depending on the physical parameters c and s. In the meantime, we use
 % Tenth order quadrature formula on unit disk from AS p. 892.

 nq = 21;
 zq( 1 ) =   0;
 zq( 2 ) =   0.4820621466885461 +   0.3502386507107874 * i;
 zq( 3 ) =   0.1841313553452870 +   0.5666980410239565 * i;
 zq( 4 ) =  -0.1841313553452870 +   0.5666980410239566 * i;
 zq( 5 ) =  -0.4820621466885460 +   0.3502386507107875 * i;
 zq( 6 ) =  -0.5958615826865181;
 zq( 7 ) =  -0.4820621466885461 +  -0.3502386507107874 * i;
 zq( 8 ) =  -0.1841313553452871 +  -0.5666980410239565 * i;
 zq( 9 ) =   0.1841313553452869 +  -0.5666980410239566 * i;
 zq( 10 ) =   0.4820621466885460 +  -0.3502386507107875 * i;
 zq( 11 ) =   0.5958615826865181;
 zq( 12 ) =   0.7436573695963749 +   0.5402987052763671 * i;
 zq( 13 ) =   0.2840518392014725 +   0.8742216692147242 * i;
 zq( 14 ) =  -0.2840518392014724 +   0.8742216692147243 * i;
 zq( 15 ) =  -0.7436573695963747 +   0.5402987052763673 * i;
 zq( 16 ) =  -0.9192110607898046;
 zq( 17 ) =  -0.7436573695963749 +  -0.5402987052763670 * i;
 zq( 18 ) =  -0.2840518392014726 +  -0.8742216692147242 * i;
 zq( 19 ) =   0.2840518392014723 +  -0.8742216692147243 * i;
 zq( 20 ) =   0.7436573695963747 +  -0.5402987052763674 * i;
 zq( 21 ) =   0.9192110607898046;
 wq( 1 ) =   0.3490658503988659;
 wq( 2 : 11 ) =   0.1610021706622441;
 wq( 12 : 21 ) =   0.1182505096568486;


 % This completes the quadrature-building section. We may want to test it:

 if( verbose > 2 )
  % Test numerical integration by integrating powers over unit disk.
  for k = 0 : nq
   sum = 0.0 + 0.0*i;
   for iq = 1 : nq
    sum = sum + wq( iq ) * ( zq( iq )' )^k * zq( iq )^k;
   end % for iq = ...
   sum = ( ( k + 1 ) / pi ) * sum;
   fprintf( 'sum( %d ) = %f + %f i should be 1 + 0i\n', k, real( sum ), imag( sum ) );
  end % for k = ...
 end % if ( verbose ...

 % Initialize and set arrays to maximum sizes required.
 
 f = 0.5 * ( 1.0 + sqrt( 1 - 4.0 * c ) );

 emin = zeros( N, 1 );
 emax = zeros( N, 1 );
 indices = zeros( N, L );

 emin( 1 : 2 ) = sort( [ sqrt(-c-f) - 0.25 , -f - 0.25 ] );
 emax( 1 : 2 ) = sort( [ f + 0.25 , -sqrt(-c-f) + 0.25 ] );
 indices( 1 : 2 ) = [ 1 ; 2 ];

 N = 2;

 % Iterate
 for level = 2 : L
  
  N2 = 2 * N;
  tmp( 1 : N ) = -1 * sqrt( emin( 1 : N ) - c * ones( N, 1 ) );
  tmp( N + 1 : N2 ) = sqrt( emin( 1 : N ) - c * ones( N, 1 ) );
  emin( 1 : N2 ) = tmp( 1 : N2 );
 
  tmp( 1 : N ) = -1 * sqrt( emax( 1 : N ) - c * ones( N, 1 ) );
  tmp( N + 1 : N2 ) = sqrt( emax( 1 : N ) - c * ones( N, 1 ) );
  emax( 1 : N2 ) = tmp( 1 : N2 );

  tmpi( 1 : N , 2 : level ) = indices( 1 : N , 1 : level - 1 );
  tmpi( N + 1 : N2 , 2 : level ) = indices( 1 : N , 1 : level - 1 );
  tmpi( 1 : N , 1 ) = 1 * ones( N, 1 );
  tmpi( N + 1 : N2 , 1 ) = 2 * ones( N, 1 );
  indices( 1 : N2, 1 : level ) = tmpi( 1 : N2, 1 : level );

  N = N2;

  % Reorder emin and emax as necessary to have emin(ib) < emax(ib) foreach ib.
  for ib = 1 : N

   if( emin( ib ) > emax( ib ) )
    swap = emax( ib );
    emax( ib ) = emin( ib );
    emin( ib ) = swap;
   end % if( emin( ib ) ...

  end % End for ib = ...

  % We may want to check inclusions at this stage and print any errors that occur.
  if( verbose > 2 )

   for ib = 1 : N
    fprintf( 'level-%d interval %d is [%f, %f]', level, ib, emin( ib ), emax( ib ) );
    fprintf( ' has history ' );
    for jb = 1 : level
      fprintf( '%d', indices( ib, jb ) );
    end % for jb ...
    fprintf( '\n' );
  
   end % for ib = ...
 
   for b = 1 : 2
    fprintf( 'g %d yields intervals\n', b );
    for ib = 1 : N
     gbmin = (-1)^b * sqrt( emin( ib ) - c );
     gbmax = (-1)^b * sqrt( emax( ib ) - c );
 
     if( gbmin > gbmax )
      swap = gbmin;
      gbmin = gbmax;
      gbmax = swap;
     end % if( gbmin ...

     fprintf( '%d [%f, %f] inside history ', ib, gbmin, gbmax );
     % Apply sigma to obtain result of mapping by g.
     sg( 1 ) = b;
     sg( 2 : level )  = indices( ib, 1: level - 1 );

     for jb = 1 : level
      fprintf( '%d', sg( jb ) );
     end % for jb = ...

     % Convert history indices into row of indices matrix.
     jb = 1;
     m = 1;
     for l = level : -1 : 1
      jb = jb + ( sg( l ) - 1 ) * m;
      m = 2 * m;
     end % for l ...
     fprintf( ' of interval %d ...', jb );

     % Now we'll actually check that interval ib is contained in jb and print messages.
     if( ( gbmin >= emin( jb ) ) & ( gbmax <= emax( jb ) ) )
      fprintf( 'OK\n' );
     else
      fprintf( 'OOPS\n' );
     end % if( ( gbmin ...

    end % for ib = ...
   end % for b = ...
  end % if( verbose ...

 end % End for level = ...

 % Now we have the endpts in pairs: form matrix of transfer operator
 % in basis consisting of complex powers centered and scaled to each interval.

 % Number of intervals is half the number of endpts.
 radius = 0.5 * ( emax - emin );
 center = emin + radius;

 if( verbose > 2 )
  fprintf( 'N = %d\n\n', N );
  fprintf( 'left endpts = %f %f\n', emin );
  fprintf( 'right endpts = %f %f\n', emax );
  fprintf( 'center = %f %f \n', center );
  fprintf( 'radius = %f %f\n', radius );
 end % if ( verbose ...

 % Loop through values of s, computing x and y
 [ m, n ] = size( sv );
 zz = zeros( m, n );

 for is = 1 : m
 for js = 1 : n
  s = sv( is, js );
  
 clear La Ld Lt;
 La = sparse( N * P, N * P );
 Ld = sparse( N * P, N * P );
 Lt = sparse( N * P, N * P );

 
 for ib = 1 : N
  for b = 1 : 2

   % Apply sigma to obtain result of mapping by g.
   sg( 1 ) = b;
   sg( 2 : L )  = indices( ib, 1: L - 1 );

   % Convert history indices into row of indices matrix.
   jb = 1;
   m = 1;
   for l = L : -1 : 1
    jb = jb + ( sg( l ) - 1 ) * m;
    m = 2 * m;
   end % for l ...

   % Form PxP block (ib, jb) by integrating basis functions against transformed
   % basis functions times power of square root over circle D_ib.
   %   a = block( ib, jb, b, P, L, N, center, radius, indices, c, s );
   % Build nonzero PxP block at location ( ib, jb ) in block matrix, by integrating
   % f_ik* vs L_b f_jl over circle D_ib for basis functions 1 <= k,l <= P on circles:
   %		
   %	f_ik(z) = ( 1/radius_ib) sqrt( (k+1) / pi ) ( ( z - center_ib ) / radius_ib )^k
   %
   % Here f_jl is transformed by L_b to yield
   % 
   %	L_b f_jl(z) = [ g_b'(z) ]^s f_jl( g_b(z) ).
   %
   % The square brackets indicate that we always take the positive sign so 
   %
   %	[ g_b'(z) ] = 1 / ( 2 * sqrt( z - c ) ).
     
   ri = radius( ib );
   ci = center( ib );
   rj = radius( jb );
   cj = center( jb );
  
   % Shift and scale quadrature points to D_i
   zqi = ci * ones( size( zq ) ) + ri * zq;
 
   % Integrate f_ik* vs L_b f_jl over D_i and accumulate in a( k, l )
   a = zeros( P, P );
   d = zeros( P, P );
   zik = zeros( P, 1 );
   zbjl = zeros( P, 1 );
 
   for iq = 1 : nq
    z = zqi( iq );
    w = wq( iq );
 
    zi = conj( zq( iq ) );
 
    gs = (4*( z - c ))^(-s/2);
    wgs = w * gs;

    % For the derivative wrt s.
    wlg = -0.5 * wgs * log(4*( z - c ));
 
    zb =(-1)^b * sqrt( z - c );
    zbj = conj( zb - cj ) / rj;
 
    % Tabulate powers. Later we should just do this once for all blocks to save time.
    zik( 1 ) = 1.0;
    for k = 1 : P - 1
     zik( k + 1 )  = zik( k ) * zi;
    end
 
    zbjl( 1 ) = 1.0;
    for l = 1 : P - 1
     zbjl( l + 1 )  = zbjl( l ) * zbj;
    end
 
    a = a + wgs * zik * zbjl';
    d = d + wlg * zik * zbjl';
   end
 
   % Perform final scaling by all the factors that don't depend on anything else.
   a = ri * a / ( rj * pi );
   d = ri * d / ( rj * pi );
  
   sqj = zeros( P, 1 );
   for j = 1 : P
    sqj( j ) = sqrt( j );
   end
   for k = 1 : P
    a( k, 1 : P ) = sqj( k ) * a( k, 1 : P );
    d( k, 1 : P ) = sqj( k ) * d( k, 1 : P );
   end
   for l = 1 : P
    a( 1 : P, l ) = sqj( l ) * a( 1 : P, l );
    d( 1 : P, l ) = sqj( l ) * d( 1 : P, l );
   end
 
   La( ( ib - 1 ) * P + 1 : ib * P, ( jb - 1 ) * P + 1 : jb * P ) = a;
   Ld( ( ib - 1 ) * P + 1 : ib * P, ( jb - 1 ) * P + 1 : jb * P ) = d;
 
  end % End for ib = ...
 end % End for jb = ...

 La = speye( N * P ) - La;

 if( verbose > 0 )
  aa = full( La );
  save 'aa.mat' aa
 end

 Lt =  - La \ Ld ;

 
 % Compute trace by hand since matlab is buggy
 zz( is, js ) = 0;
 for k = 1 : N * P
  zz( is, js ) = zz( is, js ) + Lt( k, k );
 end

 end % for is = ...
 end % for js = ...
